
#include <set>
#include <map>
#include <unistd.h>
#include "EPollExecutor.h"
#include <sys/epoll.h>

#define MAX_EVENTS_EPOLL 20

void EPollExecutor::run() {
    std::set<Socket *> sockets;
    std::map<int, Socket *> socketRef;
    std::set<Socket *> pendingEpollRegister;
    std::set<Socket *> pendingWrite;
    std::set<Socket *> pendingRead;
    std::set<Socket *> pendingClose;

    int epollfd = epoll_create(1);

    if (epollfd == -1) {
        std::cerr << "Error creating epollfd" << std::endl;
        return;
    }

    while (!stop) {
        while (sockets.size() < concurrentConnections) {
            auto *socket = new Socket(ip, port, true, tracer);
            sockets.insert(socket);
            socketRef.insert(std::make_pair(socket->fd, socket));
            pendingEpollRegister.insert(socket);
            pendingWrite.insert(socket);
        }

        for (auto *socket : pendingEpollRegister) {
            epoll_event ev{};
            ev.events = EPOLLIN | EPOLLOUT;
            ev.data.fd = socket->fd;

            int res = epoll_ctl(epollfd, EPOLL_CTL_ADD, socket->fd, &ev);

            if (res == -1) {
                std::cerr << "Error adding socket to epollfd" << std::endl;
            }
        }

        pendingEpollRegister.clear();

        epoll_event events[MAX_EVENTS_EPOLL];

        int res = epoll_wait(epollfd, events, MAX_EVENTS_EPOLL, 0);

        if (res == 0) continue;

        if (res == -1) {
            std::cerr << "epoll wait error" << std::endl;
            return;
        }

        for (auto e : events) {
            int fd = e.data.fd;
            if (socketRef.count(fd) == 0) continue;

            Socket *socket = socketRef.at(fd);

            bool isWriteable = e.events & EPOLLOUT;
            bool isReadable = e.events & EPOLLIN;

            if (isReadable && pendingRead.count(socket)) {
                if (socket->read() > 0) {
                    pendingRead.erase(socket);

                    if (connectionReuse) {
                        pendingWrite.insert(socket);
                    } else {
                        pendingClose.insert(socket);
                    }
                }
            }

            if (isWriteable && pendingWrite.count(socket)) {
                if (socket->write(new Request(delay)) > 0) {
                    pendingWrite.erase(socket);
                    pendingRead.insert(socket);
                }
            }
        }

        for (Socket *socket : pendingClose) {
            epoll_ctl(epollfd, EPOLL_CTL_DEL, socket->fd, nullptr);
            socketRef.erase(socket->fd);
            sockets.erase(socket);
            socket->close();
        }

        pendingClose.clear();
    }

    close(epollfd);

    for (Socket *socket : sockets) {
        socket->close();
    }

    socketRef.clear();
    pendingClose.clear();
    pendingRead.clear();
    pendingWrite.clear();
}