
#include "PollExecutor.h"
#include "../shared/Socket.h"
#include <set>
#include <map>
#include <poll.h>
#include <unistd.h>

void PollExecutor::run() {
    std::map<int, Socket *> socketRef;
    std::set<Socket *> sockets;
    std::set<Socket *> pendingWrite;
    std::set<Socket *> pendingRead;
    std::set<Socket *> pendingClose;

    while (!stop) {
        while (sockets.size() < concurrentConnections) {
            auto *socket = new Socket(ip, port, true, tracer);
            socketRef.insert(std::make_pair(socket->fd, socket));
            sockets.insert(socket);
            pendingWrite.insert(socket);
        }

        struct pollfd fds[concurrentConnections];

        int i = 0;
        for (auto *socket : sockets) {
            fds[i].fd = socket->fd;
            if (pendingRead.count(socket)) {
                fds[i].events = POLLIN;
            }

            if (pendingWrite.count(socket)) {
                fds[i].events = POLLOUT;
            }
            i++;
        }

        int res = poll(fds, concurrentConnections, 0);

        if (res == 0) continue;

        if (res == -1) {
            perror("Poll error.");
            exit(1);
        }

        for (int i = 0; i < concurrentConnections; i++) {
            struct pollfd poll_r = fds[i];
            int fd = poll_r.fd;
            if (socketRef.count(fd) == 0) continue;

            Socket *socket = socketRef.at(fd);
            bool isWriteable = poll_r.revents & POLLOUT;
            bool isReadable = poll_r.revents & POLLIN;

            if (isWriteable && pendingWrite.count(socket)) {
                if (socket->write(new Request(delay)) > 0) {
                    pendingWrite.erase(socket);
                    pendingRead.insert(socket);
                }
            }

            if (isReadable && pendingRead.count(socket)) {
                if (socket->read() > 0) {
                    pendingRead.erase(socket);

                    if (connectionReuse) {
                        pendingWrite.insert(socket);
                    } else {
                        pendingClose.insert(socket);
                    }
                }
            }
        }

        for (Socket *socket : pendingClose) {
            socketRef.erase(socket->fd);
            sockets.erase(socket);
            socket->close();
        }

        pendingClose.clear();
    }

    for (Socket *socket : sockets) {
        socket->close();
    }

    socketRef.clear();
    pendingClose.clear();
    pendingRead.clear();
    pendingWrite.clear();
}

PollExecutor::~PollExecutor() {

}