
#include "SelectExecutor.h"
#include <sys/select.h>
#include <sys/time.h>
#include <vector>
#include <queue>
#include <set>
#include <stdlib.h>
#include <unistd.h>
#include "../shared/Socket.h"

void SelectExecutor::run() {
    std::set<Socket *> sockets;
    std::set<Socket *> pendingWrite;
    std::set<Socket *> pendingRead;
    std::set<Socket *> pendingClose;

    while (!stop) {
        while (sockets.size() < this->concurrentConnections) {
            auto *socket = new Socket(ip, port, true, this->tracer);
            sockets.insert(socket);
            pendingWrite.insert(socket);
        }

        fd_set fds_r;
        fd_set fds_w;
        FD_ZERO(&fds_r);
        FD_ZERO(&fds_w);

        for (auto *socket : sockets) {
            if (pendingRead.count(socket)) {
                FD_SET(socket->fd, &fds_r);
            }
            if (pendingWrite.count(socket)) {
                FD_SET(socket->fd, &fds_w);
            }
        }

        timeval tv{0, 0};

        int res = select(FD_SETSIZE, &fds_r, &fds_w, nullptr, &tv);

        if (res == -1) {
            std::cerr << "Select Error" << std::endl;
            exit(1);
        }

        if (res == 0) continue;

        if (res > 0) {
            for (Socket *socket : sockets) {
                if (FD_ISSET(socket->fd, &fds_r) && pendingRead.count(socket)) {
                    if (socket->read() > 0) {
                        pendingRead.erase(socket);

                        if (connectionReuse) {
                            pendingWrite.insert(socket);
                        } else {
                            pendingClose.insert(socket);
                        }
                    }
                }

                if (FD_ISSET(socket->fd, &fds_w) && pendingWrite.count(socket)) {
                    if (socket->write(new Request(delay)) > 0) {
                        pendingWrite.erase(socket);
                        pendingRead.insert(socket);
                    }
                }
            }
        }

        for (Socket *s : pendingClose) {
            s->close();
            sockets.erase(s);
        }

        pendingClose.clear();
    }

    for (Socket *socket : sockets) {
        socket->close();
    }

    pendingClose.clear();
    pendingRead.clear();
    pendingWrite.clear();
}