#include <utility>


#include "Socket.h"
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <cstdio>
#include <string>
#include <cstring>
#include <fcntl.h>
#include "../../logger/Tracer.h"

int Socket::connect_socket() {
    struct sockaddr_in serv_addr;
    int sock = socket(AF_INET, SOCK_STREAM, 0);

    while (sock == -1) {
        std::cerr << "Error creating a socket" << std::endl;
        sock = socket(AF_INET, SOCK_STREAM, 0);
    }

    if (nio) {
        int flags = fcntl(sock, F_GETFL);

        if (flags == -1) {
            std::cerr << "Error getting file descriptor flags" << std::endl;
        }

        int result = fcntl(sock, F_SETFL, flags | O_NONBLOCK);

        if (result == -1) {
            std::cerr << "Error setting file descriptor NIO flags" << std::endl;
        }
    }

    serv_addr.sin_family = AF_INET;
    serv_addr.sin_port = htons(port);

    inet_pton(AF_INET, ip.data(), &serv_addr.sin_addr);

    int connect_response = connect(sock, (struct sockaddr *) &serv_addr, sizeof(serv_addr));

    logger->traceConnectionOpen(sock);

    if (!nio && connect_response < 0) {
        logger->traceConnectionOpenError(-1);
        std::cerr << "Problem connecting to socket" << std::endl;
        return -1;
    }

    if (nio && connect_response == -1 && errno == EINPROGRESS) {
        // NO PROBLEM
        return sock;
    }

    return sock;
}

Socket::Socket(std::string ip, int port, bool nio, Tracer* l) {
    this->ip = std::move(ip);
    this->port = port;
    this->nio = nio;
    this->logger = l;
    this->fd = connect_socket();
}

ssize_t Socket::write(Request* request) {
    const char *data = request->data.data();
    ssize_t size = send(fd, data, strlen(data), 0);
    if (size == -1) {
        std::cerr << "Error writing request" << std::endl;
    } else {
        logger->traceConnectionRequest(fd, size);
    }

    return size;
}

ssize_t Socket::read() {
    std::vector<char> bigbuf;

    ssize_t total_size = 0;

    for (;;) {
        bool escape = false;
        char buf[100];
        ssize_t result = ::read(fd, buf, 100 * sizeof(char));

        if (result > 0) {
            for (int i = 0; i < result; i++) {
                total_size++;
                if (buf[i] == '\n') {
                    escape = true;
                    break;
                }
                bigbuf.push_back(buf[i]);
            }
            logger->traceConnectionResponse(fd);
        }
        if (escape) break;
    }

    return total_size;
}

void Socket::close() {
    logger->traceConnectionClose(fd);
    ::close(fd);
}