#include "ThreadedExecutor.h"
#include "../shared/Socket.h"
#include "../../logger/Tracer.h"
#include <thread>
#include <stdlib.h>
#include <netinet/in.h>
#include <string.h>
#include <unistd.h>
#include <vector>

void ThreadedExecutor::thread_work() {
    while (!stop) {
        auto *s = new Socket(ip, port, false, tracer);

        if (s->fd < 0) {
            exit(EXIT_FAILURE);
        }

        while (!stop) {
            auto *r = new Request(this->delay);
            ssize_t write_response = s->write(r);

            if (write_response < 0) {
                std::cerr << "Problem writing to socket" << std::endl;
                exit(EXIT_FAILURE);
            }

            s->read();

            if (!connectionReuse) break;
        }

        s->close();
        usleep(10000);
    }
}

void ThreadedExecutor::fork() {
    std::thread t1([this]() {
        thread_work();
    });
    t1.detach();
}

void ThreadedExecutor::run() {
    for (int i = 0; i < concurrentConnections; ++i) {
        fork();
    }
    std::this_thread::sleep_for(std::chrono::seconds(TEST_RUN_SECONDS_DURATION));
}