
#include "Tracer.h"
#include <stdlib.h>
#include <unistd.h>
#include <chrono>
#include <fmt/core.h>
#include <thread>

using namespace std::chrono;

long Tracer::timeInMS() {
    return duration_cast<milliseconds>(
            system_clock::now().time_since_epoch()
    ).count();
}

void Tracer::traceConnectionOpen(int fd) {
    long time = timeInMS();
    enqueue(fmt::format("{},conn_open,{}", time, fd));
}

void Tracer::traceConnectionOpenError(int err) {
    long time = timeInMS();
    enqueue(fmt::format("{},conn_open_err,{}", time, err));
}

void Tracer::traceConnectionRequest(int fd, ssize_t size) {
    long time = timeInMS();
    enqueue(fmt::format("{},conn_req,{},{}", time, fd, size));
}

void Tracer::traceConnectionResponse(int fd) {
    long time = timeInMS();
    enqueue(fmt::format("{},conn_res,{}", time, fd));
}

void Tracer::traceConnectionClose(int fd) {
    long time = timeInMS();
    enqueue(fmt::format("{},conn_close,{}", time, fd));
}

void Tracer::traceCpuUserUsage(long seconds) {
    long time = timeInMS();
    enqueue(fmt::format("{},test_user_cpu,{}", time, seconds));
}

void Tracer::traceCpuKernelUsage(long seconds) {
    long time = timeInMS();
    enqueue(fmt::format("{},test_kernel_cpu,{}", time, seconds));
}

void Tracer::enqueue(std::string msg) {
    logger->info(msg);
}

Tracer::~Tracer() {
    logger->flush();
    spdlog::drop_all();
}
