
#ifndef CODE_LOGGER_H
#define CODE_LOGGER_H

#include <iostream>
#include <queue>
#include <mutex>
#include <spdlog/spdlog.h>
#include <spdlog/async.h>
#include <spdlog/sinks/basic_file_sink.h>

class Tracer {
private:
    std::shared_ptr<spdlog::logger> logger;
    long timeInMS();
    void enqueue(std::string msg);
public:
    Tracer(const std::string &description) {
        std::string t = std::to_string(timeInMS());
        std::string filename = "/tmp/" + t + "_" + description + ".trace";
        std::cout << "Init'd tracer output to " << filename.data() << std::endl;

        logger = spdlog::basic_logger_mt<spdlog::async_factory>("async_file_logger", filename);
        logger->set_pattern("%v");
    }

    void traceConnectionOpen(int fd);
    void traceConnectionOpenError(int err);
    void traceConnectionRequest(int fd, ssize_t size);
    void traceConnectionResponse(int fd);
    void traceConnectionClose(int fd);
    void traceCpuUserUsage(long seconds);
    void traceCpuKernelUsage(long seconds);

    virtual ~Tracer();
};


#endif //CODE_LOGGER_H
